# -*- coding: utf-8 -*-

from rest_framework import serializers
from thx.api.serializers import (
    ThuxCreateSerializerMixin, ThuxDisableSerializerMixin, ThuxEnableSerializerMixin, ThuxListSerializerMixin,
    ThuxPartialUpdateSerializerMixin, ThuxRetrieveSerializerMixin, ThuxUpdateSerializerMixin,
)

from ... import models


# CARRIAGE
class CarriageSerializer(serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage'
        model = models.Carriage
        fields = '__all__'


class CarriageListSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_list'
        model = models.Carriage
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class CarriageRetrieveSerializer(ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_retrieve'
        model = models.Carriage
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class CarriageCreateSerializer(ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_create'
        model = models.Carriage
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = CarriageRetrieveSerializer

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class CarriageUpdateSerializer(ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_update'
        model = models.Carriage
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = CarriageRetrieveSerializer

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class CarriagePartialUpdateSerializer(ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_partial_update'
        model = models.Carriage
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = CarriageRetrieveSerializer

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class CarriageEnableSerializer(ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_enable'
        model = models.Carriage
        fields = ()
        response_serializer = CarriageRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class CarriageDisableSerializer(ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_disable'
        model = models.Carriage
        fields = ()
        response_serializer = CarriageRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class CarriageLightSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_light'
        model = models.Carriage
        fields = ('code', 'status', 'status_display',)

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


# DELIVERY
class DeliverySerializer(serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery'
        model = models.Delivery
        fields = '__all__'


class DeliveryListSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    carriage = CarriageRetrieveSerializer()

    class Meta:
        ref_name = 'v1_carriage_delivery_list'
        model = models.Delivery
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class DeliveryRetrieveSerializer(ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    carriage = CarriageRetrieveSerializer()

    class Meta:
        ref_name = 'v1_carriage_delivery_retrieve'
        model = models.Delivery
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class DeliveryCreateSerializer(ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery_create'
        model = models.Delivery
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = DeliveryRetrieveSerializer

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class DeliveryUpdateSerializer(ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery_update'
        model = models.Delivery
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = DeliveryRetrieveSerializer

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class DeliveryPartialUpdateSerializer(ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery_partial_update'
        model = models.Delivery
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = DeliveryRetrieveSerializer

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class DeliveryEnableSerializer(ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery_enable'
        model = models.Delivery
        fields = ()
        response_serializer = DeliveryRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class DeliveryDisableSerializer(ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery_disable'
        model = models.Delivery
        fields = ()
        response_serializer = DeliveryRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class DeliveryLightSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery_light'
        model = models.Delivery
        fields = ('code', 'status', 'status_display',)

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs.get('profile')
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []
